import { useEffect, useState, useSyncExternalStore } from 'react';
import { useParams } from 'react-router-dom';
import { useTranslation } from 'react-i18next';
import { App, Table, Button, Modal, Popconfirm } from 'antd';
import { getAgencyProductExtrasAction, searchPublishedProductsAction, addProductExtraAction, delProductExtrasAction } from '@/stores/Products/Index';
import { cloneDeep, pick } from '@/utils/commons';
import SearchForm from '@/components/SearchForm';

import RequireAuth from '@/components/RequireAuth';
import { PERM_PRODUCTS_MANAGEMENT } from '@/config';
import { useProductsTypesMapVal } from '@/hooks/useProductsSets';
import { usingStorage } from '@/hooks/usingStorage';
import useProductsStore from '@/stores/Products/Index';

const NewAddonModal = ({ onPick, ...props }) => {
  // const { travel_agency_id, use_year } = useParams();
  const { t } = useTranslation();
  const { notification, message } = App.useApp();

  const [{ travel_agency_id, use_year }] = useProductsStore((state) => [state.switchParams]);

  const productsTypesMapVal = useProductsTypesMapVal();

  const [open, setOpen] = useState(false);
  const [loading, setLoading] = useState(false); // bind loading
  const [searchLoading, setSearchLoading] = useState(false);
  const [searchResult, setSearchResult] = useState([]);

  const onSearchProducts = async (values) => {
    const copyObject = cloneDeep(values);
    const { starttime, endtime, year, ...param } = copyObject;
    setSearchLoading(true);
    setSearchResult([]);
    const search_year = year || use_year;
    const result = await searchPublishedProductsAction({ ...param, use_year: search_year, travel_agency_id });
    setSearchResult(result);
    setSearchLoading(false);
  };
  const handleAddExtras = async (item) => {
    if (typeof onPick === 'function') {
      onPick(item);
    }
  };

  const searchResultColumns = [
    { key: 'ptype', dataIndex: 'type', width: '6rem', title: t('products:ProductType'), render: (text, r) => productsTypesMapVal[text]?.label || text },
    { key: 'code', dataIndex: 'code', width: '6rem', title: t('products:Code') },
    { key: 'title', dataIndex: 'title', width: '16rem', title: t('products:Title') },
    // {
    //   title: t('products:price'),
    //   dataIndex: ['quotation', '0', 'adult_cost'],
    //   width: '10rem',
    //   render: (_, { quotation }) => `${quotation[0].adult_cost} ${quotation[0].currency} / ${quotation[0].unit_name}`,
    // },
    {
      key: 'action',
      title: '',
      width: 150,
      render: (_, record) => (
        <Button className='text-primary' onClick={() => handleAddExtras(record)}>
          绑定此项目
        </Button>
      ),
    },
  ];
  const paginationProps = {
    showTotal: (total) => t('Table.Total', { total }),
  };
  return (
    <>
      <Button type='primary' onClick={() => setOpen(true)} className='mt-2'>
        {t('New')} {t('products:EditComponents.Extras')}
      </Button>

      <Modal width={'95%'} style={{ top: 20 }} open={open} title={'添加附加'} footer={false} onCancel={() => setOpen(false)} destroyOnClose>
        <SearchForm
          fieldsConfig={{
            shows: ['year', 'keyword', 'products_types', 'city'], // 'dates',
            fieldProps: {
              year: { rules: [{ required: true }] },
              keyword: { label: t('products:Title'), col: 4 },
            },
            // sort: { keyword: 100 },
          }}
          initialValue={
            {
              // dates: [dayjs().subtract(2, 'M').startOf('M'), dayjs().endOf('M')],
              // year: dayjs().add(1, 'year'),
            }
          }
          onSubmit={(err, formVal, filedsVal) => {
            onSearchProducts(formVal);
          }}
        />
        <Table
          size={'small'}
          key={'searchProductsTable'}
          rowKey={'id'}
          loading={searchLoading}
          dataSource={searchResult}
          columns={searchResultColumns}
          pagination={searchResult.length <= 10 ? false : paginationProps}
        />
      </Modal>
    </>
  );
};

/**
 *
 */
const Extras = ({ productId, onChange, ...props }) => {
  const { t } = useTranslation();
  const { notification, message } = App.useApp();

  // const { travel_agency_id, use_year } = useParams();
  const { travelAgencyId } = usingStorage();
  const [{travel_agency_id, use_year}] = useProductsStore((state) => [state.switchParams]);

  const [extrasData, setExtrasData] = useState([]);

  const handleGetAgencyProductExtras = async () => {
    setExtrasData([]);
    // console.log('handleGetAgencyProductExtras', productId);
    const data = await getAgencyProductExtrasAction({ id: productId, travel_agency_id: travel_agency_id || travelAgencyId, use_year });
    setExtrasData(data);
  };

  const handleNewAddOn = async (item) => {
    // setExtrasData(prev => [].concat(prev, [item]));
    // todo: 提交后端; 重复绑定同一个
    const _item = pick(item, ['id', 'title', 'code']);
    const newSuccess = await addProductExtraAction({ travel_agency_id, id: productId, extras: [_item] });
    newSuccess ? message.success(`${t('Success')}`) : message.error(`${t('Failed')}`);
    await handleGetAgencyProductExtras();
  }

  const handleDelAddon = async (item) => {
    const delSuccess = await delProductExtrasAction({ travel_agency_id, id: productId, del_extras_ids: [item.id] });
    delSuccess ? message.success(`${t('Success')}`) : message.error(`${t('Failed')}`);
    await handleGetAgencyProductExtras();
  };

  useEffect(() => {
    if (productId) handleGetAgencyProductExtras();

    return () => {};
  }, [productId]);

  const columns = [
    { title: t('products:Title'), dataIndex: ['info', 'title'], width: '16rem', },
    {
      title: '',
      dataIndex: 'operation',
      width: '4rem',
      render: (_, r) => (
        <Popconfirm title={t('sureDelete')} onConfirm={(e) => handleDelAddon(r.info)} okText={t('Yes')} >
          <Button size='small' type='link' danger>
            {t('Delete')}
          </Button>
        </Popconfirm>
      ),
    },
  ];

  return (
    <>
      <RequireAuth subject={PERM_PRODUCTS_MANAGEMENT}>
        <h2>{t('products:EditComponents.Extras')}</h2>
        <Table dataSource={extrasData} columns={columns} bordered pagination={false} rowKey={(r) => r.info.id} />
        <NewAddonModal onPick={handleNewAddOn} />
      </RequireAuth>
    </>
  );
};
export default Extras;
